#include "nucleo_i2c.h"

#ifndef __CC_ARM
#define __nop()  asm("nop")
#endif

static bool I2C_Start(void);
static bool I2C_Stop(void);
static bool I2C_Addr(uint8_t adr);

static void I2C_Reset(void);

#define I2C_SPEED_STANDARD(__PCLK__, __SPEED__)            (((((__PCLK__)/((__SPEED__) << 1U)) & I2C_CCR_CCR) < 4U)? 4U:((__PCLK__) / ((__SPEED__) << 1U)))
#define I2C_SPEED_FAST(__PCLK__, __SPEED__, __DUTYCYCLE__) (((__DUTYCYCLE__) == I2C_DUTYCYCLE_2)? ((__PCLK__) / ((__SPEED__) * 3U)) : (((__PCLK__) / ((__SPEED__) * 25U)) | I2C_DUTYCYCLE_16_9))
#define I2C_SPEED(__PCLK__, __SPEED__, __DUTYCYCLE__)      (((__SPEED__) <= 100000U)? (I2C_SPEED_STANDARD((__PCLK__), (__SPEED__))) : \
                                                                  ((I2C_SPEED_FAST((__PCLK__), (__SPEED__), (__DUTYCYCLE__)) & I2C_CCR_CCR) == 0U)? 1U : \
                                                                  ((I2C_SPEED_FAST((__PCLK__), (__SPEED__), (__DUTYCYCLE__))) | I2C_CCR_FS))
#define I2C_DUTYCYCLE_2                 ((uint32_t)0x00000000U)
#define I2C_DUTYCYCLE_16_9              I2C_CCR_DUTY

bool InitI2C1(i2cSpeed spd)
{
  if ((spd != i2cSpeed100k) && (spd != i2cSpeed400k))
    return false;
  Nucleo_SetPinGPIO(GPIOB, 8, ioPortAlternatePP);
  Nucleo_SetPinAFGPIO(GPIOB, 8, 4);
  Nucleo_SetPinGPIO(GPIOB, 9, ioPortAlternateOC);
  Nucleo_SetPinAFGPIO(GPIOB, 9, 4);

  if (!(RCC->APB1ENR & RCC_APB1ENR_I2C1EN))
  {
    RCC->APB1ENR |= RCC_APB1ENR_I2C1EN;
    RCC->APB1RSTR |= RCC_APB1RSTR_I2C1RST;
    RCC->APB1RSTR &= ~RCC_APB1RSTR_I2C1RST;
  }
  I2C_Reset();

  I2C1->CR1 = I2C_CR1_PE;
  I2C1->CR2 = 0;
  RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PPRE1) | RCC_CFGR_PPRE1_DIV1;
  int apbClk = SystemCoreClock;
  int apbClkMhz = apbClk / 1000000;
  I2C1->CR2 = apbClkMhz;
  I2C1->CR1 = 0;
  I2C1->TRISE = apbClkMhz + 1;
  I2C1->CCR = I2C_SPEED(apbClk, spd, 0);
  I2C1->CR1 |= I2C_CR1_ACK;
  I2C1->CR1 |= I2C_CR1_PE;
  return true;
}

static void I2C_Reset(void)
{
  uint16_t tout;
  I2C1->CR1 |= I2C_CR1_SWRST;
  for (tout = 100; tout; tout--)
    __nop();
  I2C1->CR1 = 0;
}

static const uint16_t _timeoutI2C = MAX_TIMEOUT;

static __inline uint16_t I2C_sr(void) 
{
  uint16_t sr;
  sr  = I2C1->SR1;
  sr |= I2C1->SR2 << 16;
  return (sr);
}

static bool I2C_Start(void)
{
  uint16_t w = _timeoutI2C;
  I2C1->CR1 |= I2C_CR1_START;
  while (!(I2C_sr() & I2C_SR1_SB))
  {
    if (w)
      w--;
    else
      break;
  }
  return w;
}

static bool I2C_Stop(void)
{
  uint16_t w = _timeoutI2C;
  I2C1->CR1 |= I2C_CR1_STOP;
  while (I2C_sr() & (I2C_SR2_MSL << 16))
  {
    if (w)
      w--;
    else
      break;
  }
  return w;
}

static bool I2C_Addr(uint8_t adr)
{
  uint16_t w = _timeoutI2C;
  I2C1->DR = adr;
  while(!(I2C_sr() & I2C_SR1_ADDR))
  {
    if (w)
      w--;
    else
      break;
  }
  return w;
}

static bool I2C_Write(uint8_t val)
{
  uint16_t w = _timeoutI2C;
  I2C1->DR = val;
  while (!(I2C_sr() & I2C_SR1_BTF))
  {
    if (w)
      w--;
    else
      break;
  }
  return w;
}

bool I2C1_WriteByte(uint8_t devAdr, uint8_t cr, uint8_t val)
{
  I2C_Start();
  I2C_Addr(devAdr);
  I2C_Write(cr);
  I2C_Write(val);
  I2C_Stop();
  return true;
}

void OLED_WriteCmd(uint8_t cmd)
{
	I2C1_WriteByte(SSD1306_SLAVEADR, SSD1306_COMMANDI, cmd);
}

void OLED_WriteData(uint8_t data)
{
	I2C1_WriteByte(SSD1306_SLAVEADR, SSD1306_COMMANDI, data);
}
